@@ -3,35 +3,39 @@
 # Author:  Boris Pek <tehnick-8@yandex.ru>
 # License: GPLv2 or later
 # Created: 2012-03-24
-# Updated: 2017-08-06
+# Updated: 2020-06-03
 # Version: N/A
 
+set -e
+
 export CUR_DIR="$(dirname $(realpath -s ${0}))"
 export MAIN_DIR="$(realpath -s ${CUR_DIR}/..)"
-export PSIPLUS_DIR="${MAIN_DIR}/psi-plus-snapshots"
 
-cd "${CUR_DIR}" || exit 1
+PSIPLUS_DIR="${MAIN_DIR}/psi-plus-snapshots"
+LANG_DIR="${MAIN_DIR}/psi-plus-l10n_transifex"
+
+cd "${CUR_DIR}"
 
 case "${1}" in
 "up")
     # Pulling changes from GitHub repo.
 
-    git pull --all || exit 1
+    git pull --all
 
 ;;
 "cm")
     # Creating correct git commit.
 
-    git commit -a -m 'Translations were updated from Transifex.' || exit 1
+    git commit -a -m 'Translations were updated from Transifex.'
 
 ;;
 "tag")
     # Creating correct git tag.
 
-    cd "${PSIPLUS_DIR}" || exit 1
+    cd "${PSIPLUS_DIR}"
     CUR_TAG="$(git tag -l  | sort -r -V | head -n1)"
 
-    cd "${CUR_DIR}" || exit 1
+    cd "${CUR_DIR}"
     echo "git tag \"${CUR_TAG}\""
     git tag "${CUR_TAG}"
 
@@ -43,14 +47,14 @@ case "${1}" in
 "push")
     # Pushing changes into GitHub repo.
 
-    git push || exit 1
-    git push --tags || exit 1
+    git push
+    git push --tags
 
 ;;
 "make")
     # Making precompiled localization files.
 
-    rm translations.pro
+    rm -f translations.pro
 
     echo "TRANSLATIONS = \\" >> translations.pro
     echo translations/*.ts >> translations.pro
@@ -58,19 +62,16 @@ case "${1}" in
     lrelease ./translations.pro
 
     mkdir -p out
-    mv translations/*.qm out/ || exit 1
+    mv translations/*.qm out/
 
 ;;
 "install")
-    # Installing precompiled localization files into default directory.
+    # Installing precompiled localization files into ${DESTDIR}.
 
-    if [ ${USER} != "root" ]; then
-        echo "You are not a root now!"
-        exit 1
-    fi
-
-    mkdir -p /usr/share/psi-plus/translations/
-    cp out/*.qm /usr/share/psi-plus/translations/ || exit 1
+    [ -z "${DESTDIR}" ] && DESTDIR="/usr"
+    
+    mkdir -p "${DESTDIR}/share/psi-plus/translations/"
+    cp out/*.qm "${DESTDIR}/share/psi-plus/translations/"
 
 ;;
 "tarball")
@@ -79,10 +80,10 @@ case "${1}" in
     CUR_TAG="$(git tag -l  | sort -r -V | head -n1)"
 
     rm -rf psi-plus-translations-*
-    mkdir psi-plus-translations-${CUR_TAG} || exit 1
-    cp out/*.qm psi-plus-translations-${CUR_TAG} || exit 1
+    mkdir psi-plus-translations-${CUR_TAG}
+    cp out/*.qm psi-plus-translations-${CUR_TAG}
 
-    tar -cJf psi-plus-translations-${CUR_TAG}.tar.xz psi-plus-translations-${CUR_TAG} || exit 1
+    tar -cJf psi-plus-translations-${CUR_TAG}.tar.xz psi-plus-translations-${CUR_TAG}
     echo "Tarball with precompiled translation files is ready for upload:"
     [ ! -z "$(which realpath)" ] && echo "$(realpath ${CUR_DIR}/psi-plus-translations-${CUR_TAG}.tar.xz)"
     echo "https://sourceforge.net/projects/psiplus/files/Translations/"
@@ -92,38 +93,37 @@ case "${1}" in
     # Pulling changes from Transifex.
 
     # Test Internet connection:
-    host transifex.com > /dev/null || exit 1
+    host transifex.com > /dev/null
 
-    git status || exit 1
+    git status
 
-    LANG_DIR="${MAIN_DIR}/psi-plus-l10n_transifex"
-    cd "${LANG_DIR}" || exit 1
-    tx pull -a -s || exit 1
+    cd "${LANG_DIR}"
+    tx pull -a -s
 
-    cd "${LANG_DIR}/translations/psi-plus.full/" || exit 1
+    cd "${LANG_DIR}/translations/psi-plus.full/"
     cp *.ts "${CUR_DIR}/translations/"
 
-    cd "${LANG_DIR}/translations/psi-plus.desktop-file/" || exit 1
+    cd "${LANG_DIR}/translations/psi-plus.desktop-file/"
     cp *.desktop "${CUR_DIR}/desktop-file/"
 
-    cd "${CUR_DIR}" || exit 1
-    git status || exit 1
+    cd "${CUR_DIR}"
+    git status
 
 ;;
 "tr_up")
     # Full update of localization files.
 
-    git status || exit 1
+    git status
 
     if [ -d "${PSIPLUS_DIR}" ]; then
         echo "Updating ${PSIPLUS_DIR}"
         cd "${PSIPLUS_DIR}"
-        git pull --all || exit 1
+        git pull --all
         echo;
     else
         echo "Creating ${PSIPLUS_DIR}"
         cd "${MAIN_DIR}"
-        git clone https://github.com/psi-plus/psi-plus-snapshots.git || exit 1
+        git clone https://github.com/psi-plus/psi-plus-snapshots.git
         echo;
     fi
 
@@ -133,8 +133,8 @@ case "${1}" in
     mkdir tmp
     cd tmp/
 
-    cp "${PSIPLUS_DIR}/patches"/*/*.diff ./ || exit 1
-    cp "${PSIPLUS_DIR}/patches"/*/*.patch ./ || exit 1
+    cp "${PSIPLUS_DIR}/patches"/*/*.diff ./
+    cp "${PSIPLUS_DIR}/patches"/*/*.patch ./
     PATCHES=$(ls *.diff *.patch)
     FILES="$(grep '^--- a/' ${PATCHES} | sed -e 's|^.*:--- a/\(.*\)$|\1|' | sort -u)
            $(grep '^--- psi.orig/' ${PATCHES} | sed -e 's|^.*:--- psi.orig/\(.*\)$|\1|' | sort -u)"
@@ -143,10 +143,10 @@ case "${1}" in
         mkdir -p ${DIR}
     done
     for FILE in ${FILES} ; do
-        cp -f ${PSIPLUS_DIR}/${FILE} "${FILE}" 2>/dev/null
+        cp -f ${PSIPLUS_DIR}/${FILE} "${FILE}" 2>/dev/null || true
     done
     for PATCH in ${PATCHES} ; do
-        patch -f -p1 < "${PATCH}" > applied_patches.log
+        patch -f -p1 < "${PATCH}" > applied_patches.log || true
     done
     rm ${PATCHES}
 
@@ -156,19 +156,34 @@ case "${1}" in
     # ending of magical hack
 
     cd "${CUR_DIR}"
-    rm translations.pro
+    rm -r translations.pro
 
     echo "HEADERS = \\" >> translations.pro
-    find "${PSIPLUS_DIR}/iris" "${PSIPLUS_DIR}/src" "${CUR_DIR}/tmp" -type f -name "*.h" | \
+    find "${PSIPLUS_DIR}/iris" "${PSIPLUS_DIR}/plugins" "${PSIPLUS_DIR}/src" "${CUR_DIR}/tmp" -type f -name "*.h" | \
+        grep -v "/builddir/" | \
+        grep -v "/plugins/generic/psimedia/demo" | \
+        grep -v "/plugins/generic/psimedia/gstplugin"| \
+        grep -v "/plugins/generic/psimedia/gstprovider" | \
+        grep -v "/plugins/generic/psimedia/psimedia" | \
         while read var; do echo "  ${var} \\" >> translations.pro; done
 
     echo "SOURCES = \\" >> translations.pro
-    find "${PSIPLUS_DIR}/iris" "${PSIPLUS_DIR}/src" "${CUR_DIR}/tmp" -type f -name "*.cpp" | \
+    find "${PSIPLUS_DIR}/iris" "${PSIPLUS_DIR}/plugins" "${PSIPLUS_DIR}/src" "${CUR_DIR}/tmp" -type f -name "*.cpp" | \
+        grep -v "/builddir/" | \
+        grep -v "/plugins/generic/psimedia/demo" | \
+        grep -v "/plugins/generic/psimedia/gstplugin"| \
+        grep -v "/plugins/generic/psimedia/gstprovider" | \
+        grep -v "/plugins/generic/psimedia/psimedia" | \
         while read var; do echo "  ${var} \\" >> translations.pro; done
     echo "  ${CUR_DIR}/tmp/*.cpp" >> translations.pro
 
     echo "FORMS = \\" >> translations.pro
-    find "${PSIPLUS_DIR}/iris" "${PSIPLUS_DIR}/src" "${CUR_DIR}/tmp" -type f -name "*.ui" | \
+    find "${PSIPLUS_DIR}/iris" "${PSIPLUS_DIR}/plugins" "${PSIPLUS_DIR}/src" "${CUR_DIR}/tmp" -type f -name "*.ui" | \
+        grep -v "/builddir/" | \
+        grep -v "/plugins/generic/psimedia/demo" | \
+        grep -v "/plugins/generic/psimedia/gstplugin"| \
+        grep -v "/plugins/generic/psimedia/gstprovider" | \
+        grep -v "/plugins/generic/psimedia/psimedia" | \
         while read var; do echo "  ${var} \\" >> translations.pro; done
     echo "  ${CUR_DIR}/tmp/*.ui" >> translations.pro
 
@@ -179,38 +194,37 @@ case "${1}" in
 
     cp "${PSIPLUS_DIR}"/*.desktop "${CUR_DIR}/desktop-file/"
 
-    git status || exit 1
+    git status
 
 ;;
 "tr_fu")
     # Fast update of localization files.
 
-    git status || exit 1
+    git status
 
     lupdate -verbose ./translations.pro
 
     cp "${PSIPLUS_DIR}"/*.desktop "${CUR_DIR}/desktop-file/"
 
-    git status || exit 1
+    git status
 
 ;;
 "tr_cl")
     # Cleaning update of localization files.
 
-    git status || exit 1
+    git status
 
     lupdate -verbose -no-obsolete ./translations.pro
 
     cp "${PSIPLUS_DIR}"/*.desktop "${CUR_DIR}/desktop-file/"
 
-    git status || exit 1
+    git status
 
 ;;
 "tr_push")
     # Pushing changes to Transifex.
 
-    LANG_DIR="${MAIN_DIR}/psi-plus-l10n_transifex/"
-    cd "${LANG_DIR}/translations" || exit 1
+    cd "${LANG_DIR}/translations"
 
     cp "${CUR_DIR}"/translations/*.ts psi-plus.full/
     cp "${CUR_DIR}"/desktop-file/*.desktop psi-plus.desktop-file/
@@ -220,11 +234,11 @@ case "${1}" in
         echo "<arg> is not specified!"
         exit 1
     elif [ "${2}" = "src" ] ; then
-        tx push -s || exit 1
+        tx push -s
     elif [ "${2}" = "all" ] ; then
-        tx push -s -t --skip || exit 1
+        tx push -s -t --skip
     else
-        tx push -t -l ${2} || exit 1
+        tx push -t -l ${2} --skip
     fi
 
 ;;
@@ -232,31 +246,33 @@ case "${1}" in
     # Cloning Transifex repo.
 
     if [ -d "${MAIN_DIR}/psi-plus-l10n_transifex" ]; then
-        echo "${MAIN_DIR}/psi-plus-l10n_transifex"
-        echo "directory already exists!"
+        echo "\"${MAIN_DIR}/psi-plus-l10n_transifex\" directory already exists!"
+        exit 1
     else
         echo "Creating ${MAIN_DIR}/psi-plus-l10n_transifex"
         mkdir -p "${MAIN_DIR}/psi-plus-l10n_transifex/.tx"
         cp "transifex.config" "${MAIN_DIR}/psi-plus-l10n_transifex/.tx/config"
-        cd "${MAIN_DIR}/psi-plus-l10n_transifex" || exit 1
-        tx pull -a -s || exit 1
+        cd "${MAIN_DIR}/psi-plus-l10n_transifex"
+        tx pull -a -s
     fi
 
 ;;
 "tr_sync")
     # Syncing Transifex and GitHub repos.
 
-    "${0}" up > /dev/null || exit 1
-    "${0}" tr > /dev/null || exit 1
+    "${0}" up > /dev/null
+    "${0}" tr > /dev/null
 
     if [ "$(git status | grep 'translations/' | wc -l)" -gt 0 ]; then
-        "${0}" cm || exit 1
-        "${0}" push || exit 1
+        "${0}" cm
+        "${0}" push
     fi
     echo ;
+
 ;;
 "desktop_up")
     # Update main .desktop file
+
     GENERICNAME_FULL_DATA=$(grep -r "GenericName\[" "${CUR_DIR}/desktop-file/" | grep -v '/psi.desktop:' | grep -v '/psi_en.desktop:')
     GENERICNAME_FILTERED_DATA=$(echo "${GENERICNAME_FULL_DATA}" | sed -ne 's|^.*/psi_.*.desktop:\(.*\)$|\1|p')
     GENERICNAME_SORTED_DATA=$(echo "${GENERICNAME_FILTERED_DATA}" | sort -uV)
@@ -276,6 +292,7 @@ case "${1}" in
     # Update .desktop file for English localization
     cp -f "${CUR_DIR}/desktop-file/psi.desktop" \
           "${CUR_DIR}/desktop-file/psi_en.desktop"
+
 ;;
 *)
     # Help.
